package bou.amine.apps.readerforselfossv2.repository

import bou.amine.apps.readerforselfossv2.rest.SelfossApi
import bou.amine.apps.readerforselfossv2.rest.SelfossModel
import bou.amine.apps.readerforselfossv2.service.ApiDetailsService
import bou.amine.apps.readerforselfossv2.utils.DateUtils
import bou.amine.apps.readerforselfossv2.utils.ItemType
import com.github.`ln-12`.library.ConnectivityStatus
import com.russhwolf.settings.Settings
import io.github.aakira.napier.Napier
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class Repository(private val api: SelfossApi, private val apiDetails: ApiDetailsService, val connectivityStatus: ConnectivityStatus) {
    val settings = Settings()

    var items = ArrayList<SelfossModel.Item>()
    private val isConnectionAvailable = connectivityStatus.isNetworkConnected

    var baseUrl = apiDetails.getBaseUrl()
    lateinit var dateUtils: DateUtils

    var displayedItems = ItemType.UNREAD

    var tagFilter: SelfossModel.Tag? = null
    var sourceFilter: SelfossModel.Source? = null
    var searchFilter: String? = null

    var itemsCaching = settings.getBoolean("items_caching", false)
    var offlineOverride = false

    var apiMajorVersion = 0
    var badgeUnread = 0
    set(value) {field = if (value < 0) { 0 } else { value } }
    var badgeAll = 0
    set(value) {field = if (value < 0) { 0 } else { value } }
    var badgeStarred = 0
    set(value) {field = if (value < 0) { 0 } else { value } }

    init {
        // TODO: Dispatchers.IO not available in KMM, an alternative solution should be found
        CoroutineScope(Dispatchers.Main).launch {
            updateApiVersion()
            dateUtils = DateUtils(apiMajorVersion)
            reloadBadges()
        }
    }

    suspend fun getNewerItems(): ArrayList<SelfossModel.Item> {
        // TODO: Use the updatedSince parameter
        var fetchedItems: List<SelfossModel.Item>? = null
        if (isNetworkAvailable()) {
            fetchedItems = api.getItems(
                displayedItems.type,
                settings.getString("prefer_api_items_number", "200").toInt(),
                offset = 0,
                tagFilter?.tag,
                sourceFilter?.id?.toLong(),
                searchFilter,
                null
            )
        } else {
            // TODO: Get items from the database
        }

        if (fetchedItems != null) {
            items = ArrayList(fetchedItems)
        }
        return items
    }

    suspend fun getOlderItems(): ArrayList<SelfossModel.Item> {
        var fetchedItems: List<SelfossModel.Item>? = null
        if (isNetworkAvailable()) {
            val offset = items.size
            fetchedItems = api.getItems(
                displayedItems.type,
                settings.getString("prefer_api_items_number", "200").toInt(),
                offset,
                tagFilter?.tag,
                sourceFilter?.id?.toLong(),
                searchFilter,
                null
            )
        } else {
            // TODO: Get items from the database
        }

        if (fetchedItems != null) {
            appendItems(fetchedItems)
        }
        return items
    }

    suspend fun allItems(itemType: ItemType): List<SelfossModel.Item>? {
        return if (isNetworkAvailable()) {
            api.getItems(
                itemType.type,
                200,
                0,
                tagFilter?.tag,
                sourceFilter?.id?.toLong(),
                searchFilter,
                null
            )
        } else {
            // TODO: Provide an error message
            null
        }
    }

    private fun appendItems(fetchedItems: List<SelfossModel.Item>) {
        // TODO: Store in DB if enabled by user
        val fetchedIDS = fetchedItems.map { it.id }
        val tmpItems = ArrayList(items.filterNot { it.id in fetchedIDS })
        tmpItems.addAll(fetchedItems)
        sortItems(tmpItems)
        items = tmpItems
    }

    private fun sortItems(items: ArrayList<SelfossModel.Item>) {
        items.sortByDescending { dateUtils.parseDate(it.datetime) }
    }

    suspend fun reloadBadges(): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            val response = api.stats()
            if (response != null) {
                badgeUnread = response.unread
                badgeAll = response.total
                badgeStarred = response.starred
                success = true
            }
        } else {
            // TODO: Compute badges from database
        }
        return success
    }

    suspend fun getTags(): List<SelfossModel.Tag>? {
        // TODO: Store in DB
        return if (isNetworkAvailable()) {
            api.tags()
        } else {
            // TODO: Compute from database
            null
        }
    }

    suspend fun getSpouts(): Map<String, SelfossModel.Spout>? {
        // TODO: Store in DB
        return if (isNetworkAvailable()) {
            api.spouts()
        } else {
            // TODO: Compute from database
            null
        }
    }

    suspend fun getSources(): ArrayList<SelfossModel.Source>? {
        // TODO: Store in DB
        return if (isNetworkAvailable()) {
            api.sources()
        } else {
            // TODO: Compute from database
            null
        }
    }

    suspend fun markAsRead(item: SelfossModel.Item): Boolean {
        val success = markAsReadById(item.id)

        if (success) {
            markAsReadLocally(item)
        }
        return success
    }

    suspend fun markAsReadById(id: Int): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            success = api.markAsRead(id.toString())?.isSuccess == true
        }
        return success
    }

    suspend fun unmarkAsRead(item: SelfossModel.Item): Boolean {
        val success = unmarkAsReadById(item.id)

        if (success) {
            unmarkAsReadLocally(item)
        }
        return success
    }

    suspend fun unmarkAsReadById(id: Int): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            success = api.unmarkAsRead(id.toString())?.isSuccess == true
        }
        return success
    }

    suspend fun starr(item: SelfossModel.Item): Boolean {
        val success = starrById(item.id)

        if (success) {
            starrLocally(item)
        }
        return success
    }

    suspend fun starrById(id: Int): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            success = api.starr(id.toString())?.isSuccess == true
        }
        return success
    }

    suspend fun unstarr(item: SelfossModel.Item): Boolean {
        val success = unstarrById(item.id)

        if (success) {
            unstarrLocally(item)
        }
        return success
    }

    suspend fun unstarrById(id: Int): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            success = api.unstarr(id.toString())?.isSuccess == true
        }
        return success
    }


    suspend fun markAllAsRead(items: ArrayList<SelfossModel.Item>): Boolean {
        var success = false
        if (isNetworkAvailable()) {
            success = api.markAllAsRead(items.map { it.id.toString() })?.isSuccess == true
        }

        if (success) {
            for (item in items) {
                markAsReadLocally(item)
            }
        }
        return success
    }

    private fun markAsReadLocally(item: SelfossModel.Item) {
        // TODO: Mark also in the database
        if (item.unread) {
            item.unread = false
            badgeUnread -= 1
        }
    }

    private fun unmarkAsReadLocally(item: SelfossModel.Item) {
        // TODO: Mark also in the database
        if (!item.unread) {
            item.unread = true
            badgeUnread += 1
        }
    }

    private fun starrLocally(item: SelfossModel.Item) {
        // TODO: Mark also in the database
        if (!item.starred) {
            item.starred = true
            badgeStarred += 1
        }
    }

    private fun unstarrLocally(item: SelfossModel.Item) {
        // TODO: Mark also in the database
        if (item.starred) {
            item.starred = false
            badgeStarred -= 1
        }
    }

    suspend fun createSource(
        title: String,
        url: String,
        spout: String,
        tags: String,
        filter: String
    ): Boolean {
        var response = false
        if (isNetworkAvailable()) {
            response = api.createSourceForVersion(
                title,
                url,
                spout,
                tags,
                filter,
                apiMajorVersion
            )?.isSuccess == true
        }

        return response
    }

    suspend fun deleteSource(id: Int): Boolean {
        // TODO: Store in DB
        var success = false
        if (isNetworkAvailable()) {
            val response = api.deleteSource(id)
            if (response != null) {
                success = response.isSuccess
            }
        }

        return success
    }

    suspend fun updateRemote(): Boolean {
        var response = false
        if (isConnectionAvailable.value) {
            response = api.update()?.isSuccess == true
        }
        return response
    }

    suspend fun login(): Boolean {
        var result = false
        if (isNetworkAvailable()) {
            try {
                val response = api.login()
                result = response?.isSuccess == true
            } catch (cause: Throwable) {
                Napier.e(cause.stackTraceToString(), tag = "RepositoryImpl.updateRemote")
            }
        }
        return result
    }

    fun refreshLoginInformation(url: String, login: String, password: String,
                                         httpLogin: String, httpPassword: String,
                                         isSelfSignedCert: Boolean) {
        settings.putString("url", url)
        settings.putString("login", login)
        settings.putString("httpUserName", httpLogin)
        settings.putString("password", password)
        settings.putString("httpPassword", httpPassword)
        settings.putBoolean("isSelfSignedCert", isSelfSignedCert)
        baseUrl = url
        api.refreshLoginInformation()
    }

    private suspend fun updateApiVersion() {
        apiMajorVersion = settings.getInt("apiVersionMajor", 0)

        if (isNetworkAvailable()) {
            val fetchedVersion = api.version()
            if (fetchedVersion != null) {
                apiMajorVersion = fetchedVersion.getApiMajorVersion()
                settings.putInt("apiVersionMajor", apiMajorVersion)
            }
        }
    }

    fun isNetworkAvailable() = isConnectionAvailable.value && !offlineOverride

    fun startNetwork() {
        connectivityStatus.start()
    }

    fun stopNetwork() {
        connectivityStatus.stop()
    }

    // TODO: Handle offline actions
}